@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-/* Copyright 2018-2019, Intel Corporation */
+/* Copyright 2018-2020, Intel Corporation */
 
 /**
  * @file
@@ -18,6 +18,7 @@
 #include <libpmemobj++/pext.hpp>
 #include <libpmemobj++/slice.hpp>
 #include <libpmemobj++/transaction.hpp>
+#include <libpmemobj++/utils.hpp>
 #include <libpmemobj/base.h>
 
 namespace pmem
@@ -119,7 +120,7 @@ struct array {
 		if (this == &other)
 			return *this;
 
-		transaction::run(pop, [&] {
+		flat_transaction::run(pop, [&] {
 			detail::conditional_add_to_tx(
 				this, 1, POBJ_XADD_ASSUME_INITIALIZED);
 			std::copy(other.cbegin(), other.cend(), _get_data());
@@ -150,7 +151,7 @@ struct array {
 		if (this == &other)
 			return *this;
 
-		transaction::run(pop, [&] {
+		flat_transaction::run(pop, [&] {
 			detail::conditional_add_to_tx(
 				this, 1, POBJ_XADD_ASSUME_INITIALIZED);
 			detail::conditional_add_to_tx(
@@ -587,7 +588,7 @@ struct array {
 	{
 		auto pop = _get_pool();
 
-		transaction::run(pop, [&] {
+		flat_transaction::run(pop, [&] {
 			detail::conditional_add_to_tx(
 				this, 1, POBJ_XADD_ASSUME_INITIALIZED);
 			std::fill(_get_data(), _get_data() + size(), value);
@@ -614,7 +615,7 @@ struct array {
 		if (this == &other)
 			return;
 
-		transaction::run(pop, [&] {
+		flat_transaction::run(pop, [&] {
 			detail::conditional_add_to_tx(
 				this, 1, POBJ_XADD_ASSUME_INITIALIZED);
 			detail::conditional_add_to_tx(
@@ -686,12 +687,7 @@ private:
 	pool_base
 	_get_pool() const
 	{
-		auto pop = pmemobj_pool_by_ptr(this);
-		if (pop == nullptr)
-			throw pmem::pool_error(
-				"Object outside of pmemobj pool.");
-
-		return pool_base(pop);
+		return pmem::obj::pool_by_vptr(this);
 	}
 };
 
