@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-3-Clause
-/* Copyright 2016-2019, Intel Corporation */
+/* Copyright 2016-2021, Intel Corporation */
 
 /**
  * @file
@@ -19,55 +19,30 @@
 #include <libpmemobj++/pool.hpp>
 #include <libpmemobj/tx_base.h>
 
+#ifndef LIBPMEMOBJ_CPP_FLAT_TX_USE_FAILURE_RETURN
+#define LIBPMEMOBJ_CPP_FLAT_TX_USE_FAILURE_RETURN true
+#endif
+
 namespace pmem
 {
 
-namespace obj
+namespace detail
 {
+/**
+ * A structure that checks if it is possible to snapshot the specified memory.
+ * Can have specialization.
+ */
+template <typename T>
+struct can_do_snapshot {
+	static constexpr bool value = LIBPMEMOBJ_CPP_IS_TRIVIALLY_COPYABLE(T);
+};
 
 /**
- * C++ transaction handler class.
- *
- * This class is the pmemobj transaction handler. Scoped transactions
- * are handled through two internal classes: @ref manual and
- * @ref automatic.
- * - @ref manual transactions need to be committed manually, otherwise
- *	they will be aborted on object destruction.\n
- * - @ref automatic transactions are only available in C++17. They
- *	handle transaction commit/abort automatically.
- *
- * This class also exposes a closure-like transaction API, which is the
- * preferred way of handling transactions.
- *
- * This API should NOT be mixed with C transactions API. One issue is that
- * C++ callbacks registered using transaction::register_callback() would not
- * be called if C++ transaction is created inside C transaction.
- * The same is true if user calls pmemobj_tx_set_user_data() inside a C++
- * transaction.
- *
- * The typical usage example would be:
- * @snippet doc_snippets/transaction.cpp general_tx_example
+ * Common functionality for basic_transaction and flat_transaction.
  */
-class transaction {
+template <bool is_flat>
+class transaction_base {
 public:
-	/**
-	 * C++ manual scope transaction class.
-	 *
-	 * This class is one of pmemobj transaction handlers. All
-	 * operations between creating and destroying the transaction
-	 * object are treated as performed in a transaction block and
-	 * can be rolled back. The manual transaction has to be
-	 * committed explicitly otherwise it will abort.
-	 *
-	 * The locks are held for the entire duration of the transaction. They
-	 * are released at the end of the scope, so within the `catch` block,
-	 * they are already unlocked. If the cleanup action requires access to
-	 * data within a critical section, the locks have to be manually
-	 * acquired once again.
-	 *
-	 *The typical usage example would be:
-	 * @snippet doc_snippets/transaction.cpp manual_tx_example
-	 */
 	class manual {
 	public:
 		/**
@@ -88,14 +63,19 @@ public:
 		{
 			int ret = 0;
 
-			if (pmemobj_tx_stage() == TX_STAGE_NONE) {
-				ret = pmemobj_tx_begin(pop.handle(), nullptr,
-						       TX_PARAM_CB,
-						       transaction::c_callback,
-						       nullptr, TX_PARAM_NONE);
-			} else {
+			nested = pmemobj_tx_stage() == TX_STAGE_WORK;
+
+			if (nested) {
 				ret = pmemobj_tx_begin(pop.handle(), nullptr,
 						       TX_PARAM_NONE);
+			} else if (pmemobj_tx_stage() == TX_STAGE_NONE) {
+				ret = pmemobj_tx_begin(
+					pop.handle(), nullptr, TX_PARAM_CB,
+					transaction_base::c_callback, nullptr,
+					TX_PARAM_NONE);
+			} else {
+				throw pmem::transaction_scope_error(
+					"Cannot start transaction in stage different than WORK or NONE");
 			}
 
 			if (ret != 0)
@@ -112,20 +92,26 @@ public:
 					"failed to add lock")
 					.with_pmemobj_errormsg();
 			}
+
+			set_failure_behavior();
 		}
 
 		/**
 		 * Destructor.
 		 *
-		 * End pmemobj transaction. If the transaction has not
-		 * been committed before object destruction, an abort
-		 * will be issued.
+		 * End pmemobj transaction. Abort the transaction if
+		 * the transaction is not 'flat' or this is the outermost
+		 * transaction. Commit the transaction otherwise.
 		 */
 		~manual() noexcept
 		{
 			/* normal exit or with an active exception */
-			if (pmemobj_tx_stage() == TX_STAGE_WORK)
-				pmemobj_tx_abort(ECANCELED);
+			if (pmemobj_tx_stage() == TX_STAGE_WORK) {
+				if (is_flat && nested)
+					pmemobj_tx_commit();
+				else
+					pmemobj_tx_abort(ECANCELED);
+			}
 
 			(void)pmemobj_tx_end();
 		}
@@ -149,6 +135,24 @@ public:
 		 * Deleted move assignment operator.
 		 */
 		manual &operator=(manual &&p) = delete;
+
+	private:
+		template <bool cnd = is_flat
+				  &&LIBPMEMOBJ_CPP_FLAT_TX_USE_FAILURE_RETURN>
+		typename std::enable_if<cnd>::type
+		set_failure_behavior()
+		{
+			pmemobj_tx_set_failure_behavior(POBJ_TX_FAILURE_RETURN);
+		}
+
+		template <bool cnd = is_flat
+				  &&LIBPMEMOBJ_CPP_FLAT_TX_USE_FAILURE_RETURN>
+		typename std::enable_if<!cnd>::type
+		set_failure_behavior()
+		{
+		}
+
+		bool nested = false;
 	};
 
 /*
@@ -156,25 +160,6 @@ public:
  * Test Recommendations. "|| _MSC_VER >= 1900" is a workaround.
  */
 #if __cpp_lib_uncaught_exceptions || _MSC_VER >= 1900
-	/**
-	 * C++ automatic scope transaction class.
-	 *
-	 * This class is one of pmemobj transaction handlers. All
-	 * operations between creating and destroying the transaction
-	 * object are treated as performed in a transaction block and
-	 * can be rolled back. If you have a C++17 compliant compiler,
-	 * the automatic transaction will commit and abort
-	 * automatically depending on the context of object destruction.
-	 *
-	 * The locks are held for the entire duration of the transaction. They
-	 * are released at the end of the scope, so within the `catch` block,
-	 * they are already unlocked. If the cleanup action requires access to
-	 * data within a critical section, the locks have to be manually
-	 * acquired once again.
-	 *
-	 * The typical usage example would be:
-	 * @snippet doc_snippets/transaction.cpp automatic_tx_example
-	 */
 	class automatic {
 	public:
 		/**
@@ -284,14 +269,14 @@ public:
 			int count;
 		} exceptions;
 
-		transaction::manual tx_worker;
+		manual tx_worker;
 	};
 #endif /* __cpp_lib_uncaught_exceptions */
 
 	/*
 	 * Deleted default constructor.
 	 */
-	transaction() = delete;
+	transaction_base() = delete;
 
 	/**
 	 * Default destructor.
@@ -299,7 +284,7 @@ public:
 	 * End pmemobj transaction. If the transaction has not been
 	 * committed before object destruction, an abort will be issued.
 	 */
-	~transaction() noexcept = delete;
+	~transaction_base() noexcept = delete;
 
 	/**
 	 * Manually abort the current transaction.
@@ -354,7 +339,7 @@ public:
 	POBJ_CPP_DEPRECATED static int
 	get_last_tx_error() noexcept
 	{
-		return transaction::error();
+		return error();
 	}
 
 	/**
@@ -390,70 +375,30 @@ public:
 	 */
 	template <typename... Locks>
 	static void
-	run(pool_base &pool, std::function<void()> tx, Locks &... locks)
+	run(obj::pool_base &pool, std::function<void()> tx, Locks &... locks)
 	{
-		int ret = 0;
+		manual worker(pool, locks...);
 
-		if (pmemobj_tx_stage() == TX_STAGE_NONE) {
-			ret = pmemobj_tx_begin(pool.handle(), nullptr,
-					       TX_PARAM_CB,
-					       transaction::c_callback, nullptr,
-					       TX_PARAM_NONE);
-		} else {
-			ret = pmemobj_tx_begin(pool.handle(), nullptr,
-					       TX_PARAM_NONE);
-		}
-
-		if (ret != 0)
-			throw pmem::transaction_error(
-				"failed to start transaction")
-				.with_pmemobj_errormsg();
-
-		auto err = add_lock(locks...);
-
-		if (err) {
-			pmemobj_tx_abort(err);
-			(void)pmemobj_tx_end();
-			throw pmem::transaction_error(
-				"failed to add a lock to the transaction")
-				.with_pmemobj_errormsg();
-		}
-
-		try {
-			tx();
-		} catch (manual_tx_abort &) {
-			(void)pmemobj_tx_end();
-			throw;
-		} catch (...) {
-			/* first exception caught */
-			if (pmemobj_tx_stage() == TX_STAGE_WORK)
-				pmemobj_tx_abort(ECANCELED);
-
-			/* waterfall tx_end for outer tx */
-			(void)pmemobj_tx_end();
-			throw;
-		}
+		tx();
 
 		auto stage = pmemobj_tx_stage();
 
 		if (stage == TX_STAGE_WORK) {
 			pmemobj_tx_commit();
 		} else if (stage == TX_STAGE_ONABORT) {
-			(void)pmemobj_tx_end();
 			throw pmem::transaction_error("transaction aborted");
 		} else if (stage == TX_STAGE_NONE) {
 			throw pmem::transaction_error(
 				"transaction ended prematurely");
 		}
-
-		(void)pmemobj_tx_end();
 	}
 
 	template <typename... Locks>
 	POBJ_CPP_DEPRECATED static void
-	exec_tx(pool_base &pool, std::function<void()> tx, Locks &... locks)
+	exec_tx(obj::pool_base &pool, std::function<void()> tx,
+		Locks &... locks)
 	{
-		transaction::run(pool, tx, locks...);
+		run(pool, tx, locks...);
 	}
 
 	/**
@@ -465,8 +410,8 @@ public:
 	 * supplied block of memory has to be within the pool registered in the
 	 * transaction. This function must be called during transaction. This
 	 * overload only participates in overload resolution of function
-	 * template if T satisfies requirements of
-	 * LIBPMEMOBJ_CPP_IS_TRIVIALLY_COPYABLE macro.
+	 * template if T is either a trivially copyable type or some PMDK
+	 * provided type.
 	 *
 	 * @param[in] addr pointer to the first object to be snapshotted.
 	 * @param[in] num number of elements to be snapshotted.
@@ -476,10 +421,9 @@ public:
 	 * @throw transaction_error when snapshotting failed or if function
 	 * wasn't called during transaction.
 	 */
-	template <
-		typename T,
-		typename std::enable_if<LIBPMEMOBJ_CPP_IS_TRIVIALLY_COPYABLE(T),
-					T>::type * = nullptr>
+	template <typename T,
+		  typename std::enable_if<detail::can_do_snapshot<T>::value,
+					  T>::type * = nullptr>
 	static void
 	snapshot(const T *addr, size_t num = 1)
 	{
@@ -522,7 +466,7 @@ public:
 	 * scope
 	 *
 	 * The typical usage example would be:
-	 * @snippet doc_snippets/transaction.cpp tx_callback_example
+	 * @snippet transaction/transaction.cpp tx_callback_example
 	 */
 	static void
 	register_callback(stage stg, std::function<void()> cb)
@@ -631,6 +575,293 @@ private:
 	}
 };
 
+} /* namespace detail */
+
+namespace obj
+{
+
+/**
+ * C++ transaction handler class. This class should be used with care.
+ * It's recommended to use pmem::obj::flat_transaction instead.
+ *
+ * This class is the pmemobj transaction handler. Scoped transactions
+ * are handled through two internal classes: @ref manual and
+ * @ref automatic.
+ * - @ref manual transactions need to be committed manually, otherwise
+ *	they will be aborted on object destruction.\n
+ * - @ref automatic transactions are only available in C++17. They
+ *	handle transaction commit/abort automatically.
+ *
+ * This class also exposes a closure-like transaction API, which is the
+ * preferred way of handling transactions.
+ *
+ * This API should NOT be mixed with C transactions API. One issue is that
+ * C++ callbacks registered using transaction::register_callback() would not
+ * be called if C++ transaction is created inside C transaction.
+ * The same is true if user calls pmemobj_tx_set_user_data() inside a C++
+ * transaction.
+ *
+ * The typical usage example would be:
+ * @snippet transaction/transaction.cpp general_tx_example
+ */
+class basic_transaction : public detail::transaction_base<false> {
+public:
+	/**
+	 * C++ manual scope transaction class.
+	 *
+	 * This class is one of pmemobj transaction handlers. All
+	 * operations between creating and destroying the transaction
+	 * object are treated as performed in a transaction block and
+	 * can be rolled back. The manual transaction has to be
+	 * committed explicitly otherwise it will abort.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * The typical usage example would be:
+	 * @snippet transaction/transaction.cpp manual_tx_example
+	 */
+	using manual = typename detail::transaction_base<false>::manual;
+
+/*
+ * XXX The Microsoft compiler does not follow the ISO SD-6: SG10 Feature
+ * Test Recommendations. "|| _MSC_VER >= 1900" is a workaround.
+ */
+#if __cpp_lib_uncaught_exceptions || _MSC_VER >= 1900
+	/**
+	 * C++ automatic scope transaction class.
+	 *
+	 * This class is one of pmemobj transaction handlers. All
+	 * operations between creating and destroying the transaction
+	 * object are treated as performed in a transaction block and
+	 * can be rolled back. If you have a C++17 compliant compiler,
+	 * the automatic transaction will commit and abort
+	 * automatically depending on the context of object destruction.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * The typical usage example would be:
+	 * @snippet transaction/transaction.cpp automatic_tx_example
+	 */
+	using automatic = typename detail::transaction_base<false>::automatic;
+#endif /* __cpp_lib_uncaught_exceptions */
+	/**
+	 * Execute a closure-like transaction and lock `locks`.
+	 *
+	 * The locks have to be persistent memory resident locks. An
+	 * attempt to lock the locks will be made. If any of the
+	 * specified locks is already locked, the method will block.
+	 * The locks are held until the end of the transaction. The
+	 * transaction does not have to be committed manually. Manual
+	 * aborts will end the transaction with an active exception.
+	 *
+	 * If an exception is thrown within the transaction, it gets aborted
+	 * and the exception is rethrown. Therefore extra care has to be taken
+	 * with proper error handling.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * @param[in,out] pool the pool in which the transaction will take
+	 *	place.
+	 * @param[in] tx an std::function<void ()> which will perform
+	 *	operations within this transaction.
+	 * @param[in,out] locks locks to be taken for the duration of
+	 *	the transaction.
+	 *
+	 * @throw transaction_error on any error pertaining the execution
+	 *	of the transaction.
+	 * @throw manual_tx_abort on manual transaction abort.
+	 */
+	template <typename... Locks>
+	static void
+	run(obj::pool_base &pool, std::function<void()> tx, Locks &... locks)
+	{
+		detail::transaction_base<false>::run(pool, tx, locks...);
+	}
+
+	/*
+	 * Deleted default constructor.
+	 */
+	basic_transaction() = delete;
+
+	/**
+	 * Default destructor.
+	 */
+	~basic_transaction() noexcept = delete;
+};
+
+/**
+ * C++ flat transaction handler class. This class is recommended over
+ * basic_transaction.
+ *
+ * This class is the pmemobj transaction handler. Scoped transactions
+ * are handled through two internal classes: @ref manual and
+ * @ref automatic.
+ * - @ref manual transactions need to be committed manually, otherwise
+ *	they will be aborted on object destruction.\n
+ * - @ref automatic transactions are only available in C++17. They
+ *	handle transaction commit/abort automatically.
+ *
+ * This class also exposes a closure-like transaction API, which is the
+ * preferred way of handling transactions.
+ *
+ * This API should NOT be mixed with C transactions API. One issue is that
+ * C++ callbacks registered using transaction::register_callback() would not
+ * be called if C++ transaction is created inside C transaction.
+ * The same is true if user calls pmemobj_tx_set_user_data() inside a C++
+ * transaction.
+ *
+ * **Unlike basic_transaction, flat_transaction does not abort
+ * automatically in case of transactional functions (like
+ * make_persistent) failures. Instead, abort will happen only if an
+ * exception is not caught before the outermost transaction ends.**
+ *
+ * The typical usage example would be:
+ * @snippet transaction/transaction.cpp tx_flat_example
+ * @snippet transaction/transaction.cpp tx_nested_struct_example
+ *
+ */
+class flat_transaction : public detail::transaction_base<true> {
+public:
+	/**
+	 * C++ manual scope transaction class.
+	 *
+	 * This class is one of pmemobj transaction handlers. All
+	 * operations between creating and destroying the transaction
+	 * object are treated as performed in a transaction block and
+	 * can be rolled back. The manual transaction has to be
+	 * committed explicitly in the outer most transaction - otherwise it
+	 * will abort. Calling commit() in inner transactions is optional.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * The typical usage example would be:
+	 * @snippet transaction/transaction.cpp manual_flat_tx_example
+	 */
+	using manual = typename detail::transaction_base<true>::manual;
+
+/*
+ * XXX The Microsoft compiler does not follow the ISO SD-6: SG10 Feature
+ * Test Recommendations. "|| _MSC_VER >= 1900" is a workaround.
+ */
+#if __cpp_lib_uncaught_exceptions || _MSC_VER >= 1900
+	/**
+	 * C++ automatic scope transaction class.
+	 *
+	 * This class is one of pmemobj transaction handlers. All
+	 * operations between creating and destroying the transaction
+	 * object are treated as performed in a transaction block and
+	 * can be rolled back. If you have a C++17 compliant compiler,
+	 * the automatic transaction will commit and abort
+	 * automatically depending on the context of object destruction.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * The typical usage example would be:
+	 * @snippet transaction/transaction.cpp automatic_tx_example
+	 */
+	using automatic = typename detail::transaction_base<true>::automatic;
+#endif /* __cpp_lib_uncaught_exceptions */
+	/**
+	 * Execute a closure-like transaction and lock `locks`.
+	 *
+	 * The locks have to be persistent memory resident locks. An
+	 * attempt to lock the locks will be made. If any of the
+	 * specified locks is already locked, the method will block.
+	 * The locks are held until the end of the transaction. The
+	 * transaction does not have to be committed manually. Manual
+	 * aborts will end the transaction with an active exception.
+	 *
+	 * If an exception is thrown within the transaction, it gets propagated
+	 * to the outer most transaction. If the exception is not caught, it
+	 * will result in a transaction abort.
+	 *
+	 * The locks are held for the entire duration of the transaction. They
+	 * are released at the end of the scope, so within the `catch` block,
+	 * they are already unlocked. If the cleanup action requires access to
+	 * data within a critical section, the locks have to be manually
+	 * acquired once again.
+	 *
+	 * @param[in,out] pool the pool in which the transaction will take
+	 *	place.
+	 * @param[in] tx an std::function<void ()> which will perform
+	 *	operations within this transaction.
+	 * @param[in,out] locks locks to be taken for the duration of
+	 *	the transaction.
+	 *
+	 * @throw transaction_error on any error pertaining the execution
+	 *	of the transaction.
+	 * @throw manual_tx_abort on manual transaction abort.
+	 */
+	template <typename... Locks>
+	static void
+	run(obj::pool_base &pool, std::function<void()> tx, Locks &... locks)
+	{
+		detail::transaction_base<true>::run(pool, tx, locks...);
+	}
+
+	/*
+	 * Deleted default constructor.
+	 */
+	flat_transaction() = delete;
+
+	/**
+	 * Default destructor.
+	 */
+	~flat_transaction() noexcept = delete;
+};
+
+#ifdef LIBPMEMOBJ_CPP_USE_FLAT_TRANSACTION
+/**
+ * By default, pmem::obj::transaction is an alias to
+ * pmem::obj::basic_transaction. To change it to pmem::obj::flat_transaction
+ * define LIBPMEMOBJ_CPP_USE_FLAT_TRANSACTION macro.
+ *
+ * To see what is the difference between the two please look at the examples for
+ * flat tx:
+ * @snippet transaction/transaction.cpp tx_flat_example
+ * @snippet transaction/transaction.cpp tx_nested_struct_example
+ *
+ * and basic tx:
+ * @snippet transaction/transaction.cpp manual_tx_example
+ */
+using transaction = flat_transaction;
+#else
+/**
+ * By default, pmem::obj::transaction is an alias to
+ * pmem::obj::basic_transaction. To change it to pmem::obj::flat_transaction
+ * define LIBPMEMOBJ_CPP_USE_FLAT_TRANSACTION macro.
+ *
+ * To see what is the difference between the two please look at the examples for
+ * flat tx:
+ * @snippet transaction/transaction.cpp tx_flat_example
+ * @snippet transaction/transaction.cpp tx_nested_struct_example
+ *
+ * and basic tx:
+ * @snippet transaction/transaction.cpp manual_tx_example
+ */
+using transaction = basic_transaction;
+#endif
+
 } /* namespace obj */
 
 } /* namespace pmem */
