@@ -7,26 +7,22 @@
 //
 //===----------------------------------------------------------------------===//
 //
-// Copyright 2018, Intel Corporation
+// Copyright 2018-2020, Intel Corporation
 //
 // Modified to test pmem::obj containers
 //
 
 #include "unittest.hpp"
 
-#include <libpmemobj++/experimental/array.hpp>
-
-namespace pmem_exp = pmem::obj::experimental;
+#include <libpmemobj++/container/array.hpp>
 
 int
 main()
 {
 	{
-		START();
-
 		typedef double T;
-		typedef pmem_exp::array<const T, 0> C;
-		C c = {};
+		typedef pmem::obj::array<const T, 0> C;
+		C c = {{}};
 		// expected-error-re@array:* {{static_assert failed
 		// {{.*}}"cannot fill zero-sized array of type 'const T'"}}
 		c.fill(5.5); // expected-note {{requested here}}
