@@ -1,34 +1,5 @@
-/*
- * Copyright 2015-2019, Intel Corporation
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *
- *     * Neither the name of the copyright holder nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-3-Clause
+/* Copyright 2015-2020, Intel Corporation */
 
 /*
  * obj_cpp_pool.c -- cpp pool implementation test
@@ -155,11 +126,9 @@ get_root_closed()
 
 } /* namespace */
 
-int
-main(int argc, char *argv[])
+static void
+test(int argc, char *argv[])
 {
-	START();
-
 	if (argc < 4)
 		UT_FATAL("usage: %s op path layout [poolsize mode]", argv[0]);
 
@@ -169,7 +138,7 @@ main(int argc, char *argv[])
 
 	if (strcmp(argv[3], "EMPTY") == 0)
 		layout = "";
-	else if (strcmp(argv[3], "NULL") != 0)
+	else
 		layout = argv[3];
 
 	switch (argv[1][0]) {
@@ -178,7 +147,6 @@ main(int argc, char *argv[])
 				MB; /* in megabytes */
 			mode = static_cast<unsigned>(
 				std::stoul(argv[5], nullptr, 8));
-
 			pool_create(argv[2], layout, poolsize, mode);
 			break;
 		case 'o':
@@ -189,7 +157,6 @@ main(int argc, char *argv[])
 				MB; /* in megabytes */
 			mode = static_cast<unsigned>(
 				std::stoul(argv[5], nullptr, 8));
-
 			double_close(argv[2], layout, poolsize, mode);
 			break;
 		case 'i':
@@ -198,6 +165,10 @@ main(int argc, char *argv[])
 		default:
 			UT_FATAL("unknown operation");
 	}
+}
 
-	return 0;
+int
+main(int argc, char *argv[])
+{
+	return run_test([&] { test(argc, argv); });
 }
